@@ -273,6 +273,12 @@ int drbd_bm_init(drbd_dev *mdev)
 	return 0;
 }
 
+sector_t drbd_bm_capacity(drbd_dev *mdev)
+{
+	ERR_IF(!mdev->bitmap) return 0;
+	return mdev->bitmap->bm_dev_capacity;
+}
+
 /* called on driver unload. TODO: call when a device is destroyed.
  */
 void drbd_bm_cleanup(drbd_dev *mdev)
@@ -355,7 +361,7 @@ int drbd_bm_resize(drbd_dev *mdev, sector_t capacity)
 	unsigned long bits, bytes, words, *nbm, *obm = 0;
 	int err = 0, growing;
 
-	D_BUG_ON(!b);
+	ERR_IF(!b) return -ENOMEM;
 	MUST_BE_LOCKED();
 
 	ERR_IF (down_trylock(&b->bm_change)) {
@@ -456,7 +462,7 @@ unsigned long drbd_bm_total_weight(drbd_dev *mdev)
 	unsigned long s;
 	unsigned long flags;
 
-	D_BUG_ON(!(b && b->bm));
+	ERR_IF(!b) return 0;
 	// MUST_BE_LOCKED(); well. yes. but ...
 
 	spin_lock_irqsave(&b->bm_lock,flags);
@@ -469,7 +475,7 @@ unsigned long drbd_bm_total_weight(drbd_dev *mdev)
 size_t drbd_bm_words(drbd_dev *mdev)
 {
 	struct drbd_bitmap *b = mdev->bitmap;
-	D_BUG_ON(!(b && b->bm));
+	ERR_IF(!b) return 0;
 
 	/* FIXME
 	 * actually yes. really. otherwise it could just change its size ...
@@ -491,7 +497,8 @@ void drbd_bm_merge_lel( drbd_dev *mdev, size_t offset, size_t number,
 	unsigned long word, bits;
 	size_t n = number;
 
-	D_BUG_ON(!(b && b->bm));
+	ERR_IF(!b) return;
+	ERR_IF(!b->bm) return;
 	D_BUG_ON(offset        >= b->bm_words);
 	D_BUG_ON(offset+number >  b->bm_words);
 	D_BUG_ON(number > PAGE_SIZE/sizeof(long));
@@ -530,7 +537,8 @@ void drbd_bm_set_lel( drbd_dev *mdev, size_t offset, size_t number,
 	unsigned long word, bits;
 	size_t n = number;
 
-	D_BUG_ON(!(b && b->bm));
+	ERR_IF(!b) return;
+	ERR_IF(!b->bm) return;
 	D_BUG_ON(offset        >= b->bm_words);
 	D_BUG_ON(offset+number >  b->bm_words);
 	D_BUG_ON(number > PAGE_SIZE/sizeof(long));
@@ -567,7 +575,8 @@ void drbd_bm_get_lel( drbd_dev *mdev, size_t offset, size_t number,
 	struct drbd_bitmap *b = mdev->bitmap;
 	unsigned long *bm;
 
-	D_BUG_ON(!(b && b->bm));
+	ERR_IF(!b) return;
+	ERR_IF(!b->bm) return;
 	if ( (offset        >= b->bm_words) ||
 	     (offset+number >  b->bm_words) ||
 	     (number > PAGE_SIZE/sizeof(long)) ||
@@ -593,7 +602,8 @@ void drbd_bm_get_lel( drbd_dev *mdev, size_t offset, size_t number,
 void drbd_bm_set_all(drbd_dev *mdev)
 {
 	struct drbd_bitmap *b = mdev->bitmap;
-	D_BUG_ON(!(b && b->bm));
+	ERR_IF(!b) return;
+	ERR_IF(!b->bm) return;
 
 	MUST_BE_LOCKED();
 
@@ -737,7 +747,8 @@ void drbd_bm_write(struct Drbd_Conf *mdev)
 void drbd_bm_clear_all(drbd_dev *mdev)
 {
 	struct drbd_bitmap *b = mdev->bitmap;
-	D_BUG_ON(!(b && b->bm));
+	ERR_IF(!b) return;
+	ERR_IF(!b->bm) return;
 
 	MUST_BE_LOCKED();						\
 
@@ -751,7 +762,7 @@ void drbd_bm_clear_all(drbd_dev *mdev)
 void drbd_bm_reset_find(drbd_dev *mdev)
 {
 	struct drbd_bitmap *b = mdev->bitmap;
-	D_BUG_ON(!(b && b->bm));
+	ERR_IF(!b) return;
 
 	MUST_BE_LOCKED();
 
@@ -771,7 +782,8 @@ unsigned long drbd_bm_find_next(drbd_dev *mdev)
 {
 	struct drbd_bitmap *b = mdev->bitmap;
 	unsigned long i = -1UL;
-	D_BUG_ON(!(b && b->bm));
+	ERR_IF(!b) return i;
+	ERR_IF(!b->bm) return i;
 
 	spin_lock_irq(&b->bm_lock);
 	BM_PARANOIA_CHECK();
@@ -804,7 +816,8 @@ int drbd_bm_set_bit(drbd_dev *mdev, const unsigned long bitnr)
 {
 	struct drbd_bitmap *b = mdev->bitmap;
 	int i;
-	D_BUG_ON(!(b && b->bm));
+	ERR_IF(!b) return 1;
+	ERR_IF(!b->bm) return 1;
 
 /*
  * only called from drbd_set_out_of_sync.
@@ -837,10 +850,12 @@ int drbd_bm_set_bit(drbd_dev *mdev, const unsigned long bitnr)
 int drbd_bm_clear_bit(drbd_dev *mdev, const unsigned long bitnr)
 {
 	struct drbd_bitmap *b = mdev->bitmap;
+	unsigned long flags;
 	int i;
-	D_BUG_ON(!(b && b->bm));
+	ERR_IF(!b) return 0;
+	ERR_IF(!b->bm) return 0;
 
-	spin_lock_irq(&b->bm_lock);
+	spin_lock_irqsave(&b->bm_lock,flags);
 	BM_PARANOIA_CHECK();
 	MUST_NOT_BE_LOCKED();
 	ERR_IF (bitnr >= b->bm_bits) {
@@ -850,7 +865,7 @@ int drbd_bm_clear_bit(drbd_dev *mdev, const unsigned long bitnr)
 		i = (0 != __test_and_clear_bit(bitnr, b->bm));
 		b->bm_set -= i;
 	}
-	spin_unlock_irq(&b->bm_lock);
+	spin_unlock_irqrestore(&b->bm_lock,flags);
 
 	/* clearing bits should only take place when sync is in progress!
 	 * this is only called from drbd_set_in_sync.
@@ -871,7 +886,8 @@ int drbd_bm_test_bit(drbd_dev *mdev, const unsigned long bitnr)
 {
 	struct drbd_bitmap *b = mdev->bitmap;
 	int i;
-	D_BUG_ON(!(b && b->bm));
+	ERR_IF(!b) return 0;
+	ERR_IF(!b->bm) return 0;
 
 	spin_lock_irq(&b->bm_lock);
 	BM_PARANOIA_CHECK();
@@ -905,7 +921,8 @@ int drbd_bm_e_weight(drbd_dev *mdev, unsigned long enr)
 	int count, s, e;
 	unsigned long flags;
 
-	D_BUG_ON(!(b && b->bm));
+	ERR_IF(!b) return 0;
+	ERR_IF(!b->bm) return 0;
 	spin_lock_irqsave(&b->bm_lock,flags);
 	BM_PARANOIA_CHECK();
 
@@ -932,7 +949,8 @@ unsigned long drbd_bm_ALe_set_all(drbd_dev *mdev, unsigned long al_enr)
 	struct drbd_bitmap *b = mdev->bitmap;
 	unsigned long weight;
 	int count, s, e;
-	D_BUG_ON(!(b && b->bm));
+	ERR_IF(!b) return 0;
+	ERR_IF(!b->bm) return 0;
 
 	MUST_BE_LOCKED();
 
