@@ -76,7 +76,15 @@ extern asmlinkage int sys_ioctl(unsigned int fd, unsigned int cmd, unsigned long
 # endif
 #else
 # ifdef CONFIG_COMPAT
-#  include <linux/ioctl32.h>
+#  if LINUX_VERSION_CODE > KERNEL_VERSION(2,6,10)
+    /* FIXME on which thing could we test instead of the KERNEL_VERSION
+     * again?  register_ioctl32_conversion was deprecated in 2.6.10, got
+     * "officially" deprecated somewhen in 2.6.12, and removed in 2.6.14.
+     * so lets assume all vendor kernels did the transition.  */
+#    define HAVE_COMPAT_IOCTL_MEMBER
+#  else
+#   include <linux/ioctl32.h>
+#  endif
 # endif
 #endif
 
@@ -103,13 +111,12 @@ MODULE_LICENSE("GPL");
 MODULE_PARM_DESC(use_nbd_major, "DEPRECATED! use nbd device major nr (43) "
 		                "instead of the default " __stringify(LANANA_DRBD_MAJOR) );
 MODULE_PARM_DESC(minor_count, "Maximum number of drbd devices (1-255)");
-MODULE_PARM_DESC(disable_io_hints, "Necessary if the loopback network device is used for DRBD" );
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2,5,0)
 MODULE_PARM(use_nbd_major,"i");
 MODULE_PARM(minor_count,"i");
-MODULE_PARM(disable_io_hints,"i");
 #else
 #include <linux/moduleparam.h>
+MODULE_PARM_DESC(disable_bd_claim, "DONT USE! disables block device claiming" );
 /*
  * please somebody explain to me what the "perm" of the module_param
  * macro is good for (yes, permission for it in the "driverfs", but what
@@ -126,7 +133,7 @@ MODULE_PARM(disable_io_hints,"i");
  */
 module_param(use_nbd_major,   bool,0);
 module_param(minor_count,      int,0);
-module_param(disable_io_hints,bool,0);
+module_param(disable_bd_claim,bool,0);
 #endif
 
 // module parameter, defined
@@ -137,8 +144,7 @@ int minor_count = 2;
 #else
 int minor_count = 8;
 #endif
-// FIXME disable_io_hints shall die
-int disable_io_hints = 0;
+int disable_bd_claim = 0;
 
 // devfs name
 char* drbd_devfs_name = "drbd";
@@ -165,7 +171,10 @@ STATIC struct block_device_operations drbd_ops = {
 #endif
 	.open =    drbd_open,
 	.release = drbd_close,
-	.ioctl =   drbd_ioctl
+	.ioctl =   drbd_ioctl,
+#ifdef HAVE_COMPAT_IOCTL_MEMBER
+	.compat_ioctl = drbd_compat_ioctl,
+#endif
 };
 
 #define ARRY_SIZE(A) (sizeof(A)/sizeof(A[0]))
@@ -486,8 +495,10 @@ STATIC int drbd_thread_setup(void* arg)
 	drbd_daemonize();
 	D_ASSERT(get_t_state(thi) == Running);
 	D_ASSERT(thi->task == NULL);
+	spin_lock(&thi->t_lock);
 	thi->task = current;
 	smp_mb();
+	spin_unlock(&thi->t_lock);
 	complete(&thi->startstop); // notify: thi->task is set.
 
 	retval = thi->function(thi);
@@ -1325,10 +1336,6 @@ void drbd_set_defaults(drbd_dev *mdev)
 {
 	mdev->flags = 1<<DISKLESS;
 
-	/* If the UNPLUG_QUEUED flag is set but it is not
-	   actually queued the functionality is completely disabled */
-	if (disable_io_hints) mdev->flags |= 1<<UNPLUG_QUEUED;
-
 	mdev->sync_conf.rate       = 250;
 	mdev->sync_conf.al_extents = 127; // 512 MB active set
 	mdev->state                = Secondary;
@@ -1476,6 +1483,7 @@ void drbd_mdev_cleanup(drbd_dev *mdev)
 	mdev->rs_mark_time = 0;
 	mdev->send_task    = NULL;
 	drbd_set_my_capacity(mdev,0);
+	drbd_bm_resize(mdev,0);
 
 	// just in case
 	drbd_free_resources(mdev);
@@ -1647,6 +1655,7 @@ ONLY_IN_26(
 		drbd_destroy_mempools();
 	}
 
+#ifndef HAVE_COMPAT_IOCTL_MEMBER
 #if defined(CONFIG_PPC64) || defined(CONFIG_SPARC64) || defined(CONFIG_X86_64)
 	lock_kernel();
 	unregister_ioctl32_conversion(DRBD_IOCTL_GET_VERSION);
@@ -1664,6 +1673,7 @@ ONLY_IN_26(
 	unregister_ioctl32_conversion(DRBD_IOCTL_UNCONFIG_DISK);
 	unlock_kernel();
 #endif
+#endif
 
 NOT_IN_26(
 	blksize_size[MAJOR_NR] = NULL;
@@ -1692,6 +1702,7 @@ int __init drbd_init(void)
 	int i,err;
 
 #if 0
+#warning "DEBUGGING"
 /* I am too lazy to calculate this by hand	-lge
  */
 #define SZO(x) printk(KERN_ERR "sizeof(" #x ") = %d\n", sizeof(x))
@@ -1808,6 +1819,8 @@ int __init drbd_init(void)
 		if (bd_claim(mdev->this_bdev,drbd_sec_holder)) {
 			// Initial we are Secondary -> should claim myself.
 			WARN("Could not bd_claim() myself.");
+		} else if (disable_bd_claim) {
+			bd_release(mdev->this_bdev);
 		}
 
 		blk_queue_make_request(q,drbd_make_request_26);
@@ -1852,13 +1865,14 @@ NOT_IN_26(
 	/*
 	 * register with procfs
 	 */
-	// XXX maybe move to a seq_file interface
-	drbd_proc = create_proc_read_entry("drbd", 0, &proc_root,
-					   drbd_proc_get_info, NULL);
+	drbd_proc = create_proc_entry("drbd",  S_IFREG | S_IRUGO , &proc_root);
+
 	if (!drbd_proc)	{
 		printk(KERN_ERR DEVICE_NAME": unable to register proc file\n");
 		goto Enomem;
 	}
+	
+	drbd_proc->proc_fops = &drbd_proc_fops;
 	drbd_proc->owner = THIS_MODULE;
 #else
 # error "Currently drbd depends on the proc file system (CONFIG_PROC_FS)"
@@ -1878,6 +1892,7 @@ NOT_IN_26(
 
 	NOT_IN_26(blk_queue_make_request(BLK_DEFAULT_QUEUE(MAJOR_NR),drbd_make_request_24);)
 
+#ifndef HAVE_COMPAT_IOCTL_MEMBER
 #if defined(CONFIG_PPC64) || defined(CONFIG_SPARC64) || defined(CONFIG_X86_64)
 	// tell the kernel that we think our ioctls are 64bit clean
 	lock_kernel();
@@ -1895,6 +1910,7 @@ NOT_IN_26(
 	register_ioctl32_conversion(DRBD_IOCTL_WAIT_SYNC,NULL);
 	register_ioctl32_conversion(DRBD_IOCTL_UNCONFIG_DISK,NULL);
 	unlock_kernel();
+#endif
 #endif
 
 	printk(KERN_INFO DEVICE_NAME ": initialised. "
@@ -1974,7 +1990,7 @@ struct meta_data_on_disk {
 	u32 al_offset;         // offset to this block
 	u32 al_nr_extents;     // important for restoring the AL
 	u32 bm_offset;         // offset to the bitmap, from here
-};
+} __attribute((packed));
 
 /*
 
