@@ -35,12 +35,46 @@
 #include <linux/file.h>
 #include <linux/slab.h>
 #include <linux/proc_fs.h>
+#include <linux/seq_file.h>
 #include <linux/drbd.h>
 #include "drbd_int.h"
 
-int drbd_proc_get_info(char *, char **, off_t, int, int *, void *);
+STATIC int drbd_proc_open(struct inode *inode, struct file *file);
+STATIC int drbd_seq_show(struct seq_file *seq, void *v);
+
 
 struct proc_dir_entry *drbd_proc;
+struct file_operations drbd_proc_fops = {
+	.owner		= THIS_MODULE,
+	.open		= drbd_proc_open,
+	.read		= seq_read,
+	.llseek		= seq_lseek,
+	.release	= seq_release,
+};
+
+// We ommit single_open and single_release, since that is only available
+// after 2.4.23
+static void *single_start(struct seq_file *p, loff_t *pos)
+{
+	return NULL + (*pos == 0);
+}
+
+static void *single_next(struct seq_file *p, void *v, loff_t *pos)
+{
+	++*pos;
+	return NULL;
+}
+
+static void single_stop(struct seq_file *p, void *v)
+{
+}
+
+struct seq_operations drbd_proc_seq_ops = {
+	.start		= single_start,
+	.next		= single_next,
+	.stop		= single_stop,
+	.show		= drbd_seq_show,
+};
 
 /*lge
  * progress bars shamelessly adapted from driver/md/md.c
@@ -48,9 +82,8 @@ struct proc_dir_entry *drbd_proc;
  *	[=====>..............] 33.5% (23456/123456)
  *	finish: 2:20:20 speed: 6,345 (6,456) K/sec
  */
-STATIC int drbd_syncer_progress(struct Drbd_Conf* mdev,char *buf)
+STATIC void drbd_syncer_progress(struct Drbd_Conf* mdev, struct seq_file *seq)
 {
-	int sz = 0;
 	unsigned long res , db, dt, dbdt, rt, rs_left;
 
 	/* the whole sector_div thingy was wrong (did overflow,
@@ -82,23 +115,23 @@ STATIC int drbd_syncer_progress(struct Drbd_Conf* mdev,char *buf)
 	}
 	{
 		int i, y = res/50, x = 20-y;
-		sz += sprintf(buf + sz, "\t[");
+		seq_printf(seq, "\t[");
 		for (i = 1; i < x; i++)
-			sz += sprintf(buf + sz, "=");
-		sz += sprintf(buf + sz, ">");
+			seq_printf(seq, "=");
+		seq_printf(seq, ">");
 		for (i = 0; i < y; i++)
-			sz += sprintf(buf + sz, ".");
-		sz += sprintf(buf + sz, "] ");
+			seq_printf(seq, ".");
+		seq_printf(seq, "] ");
 	}
 	res = 1000L - res;
-	sz+=sprintf(buf+sz,"sync'ed:%3lu.%lu%% ", res / 10, res % 10);
+	seq_printf(seq,"sync'ed:%3lu.%lu%% ", res / 10, res % 10);
 	/* if more than 1 GB display in MB */
 	if (mdev->rs_total > 0x100000L) {
-		sz+=sprintf(buf+sz,"(%lu/%lu)M\n\t",
+		seq_printf(seq,"(%lu/%lu)M\n\t",
 			    (unsigned long) Bit2KB(rs_left) >> 10,
 			    (unsigned long) Bit2KB(mdev->rs_total) >> 10 );
 	} else {
-		sz+=sprintf(buf+sz,"(%lu/%lu)K\n\t",
+		seq_printf(seq,"(%lu/%lu)K\n\t",
 			    (unsigned long) Bit2KB(rs_left),
 			    (unsigned long) Bit2KB(mdev->rs_total) );
 	}
@@ -113,20 +146,28 @@ STATIC int drbd_syncer_progress(struct Drbd_Conf* mdev,char *buf)
 	 * rt: remaining time
 	 */
 	dt = (jiffies - mdev->rs_mark_time) / HZ;
+
+	if (dt > 20) {
+		/* if we made no update to rs_mark_time for too long,
+		 * we are stalled. show that. */
+		seq_printf(seq, "stalled\n");
+		return;
+	}
+
 	if (!dt) dt++;
 	db = mdev->rs_mark_left - rs_left;
 	rt = (dt * (rs_left / (db/100+1)))/100; /* seconds */
 
-	sz += sprintf(buf + sz, "finish: %lu:%02lu:%02lu",
+	seq_printf(seq, "finish: %lu:%02lu:%02lu",
 		rt / 3600, (rt % 3600) / 60, rt % 60);
 
 	/* current speed average over (SYNC_MARKS * SYNC_MARK_STEP) jiffies */
 	dbdt = Bit2KB(db/dt);
 	if (dbdt > 1000)
-		sz += sprintf(buf + sz, " speed: %ld,%03ld",
+		seq_printf(seq, " speed: %ld,%03ld",
 			dbdt/1000,dbdt % 1000);
 	else
-		sz += sprintf(buf + sz, " speed: %ld", dbdt);
+		seq_printf(seq, " speed: %ld", dbdt);
 
 	/* mean speed since syncer started
 	 * we do account for PausedSync periods */
@@ -135,14 +176,12 @@ STATIC int drbd_syncer_progress(struct Drbd_Conf* mdev,char *buf)
 	db = mdev->rs_total - rs_left;
 	dbdt = Bit2KB(db/dt);
 	if (dbdt > 1000)
-		sz += sprintf(buf + sz, " (%ld,%03ld)",
+		seq_printf(seq, " (%ld,%03ld)",
 			dbdt/1000,dbdt % 1000);
 	else
-		sz += sprintf(buf + sz, " (%ld)", dbdt);
+		seq_printf(seq, " (%ld)", dbdt);
 
-	sz += sprintf(buf+sz," K/sec\n");
-
-	return sz;
+	seq_printf(seq," K/sec\n");
 }
 
 const char* cstate_to_name(Drbd_CState s) {
@@ -183,16 +222,14 @@ const char* nodestate_to_name(Drbd_State s) {
 		              : state_names[s];
 }
 
-/* FIXME we should use snprintf, we only have guaranteed room for one page...
- * we should eventually use seq_file for this */
-int drbd_proc_get_info(char *buf, char **start, off_t offset,
-		       int len, int *unused, void *data)
+
+STATIC int drbd_seq_show(struct seq_file *seq, void *v)
 {
-	int rlen, i;
+	int i;
 	const char *sn;
 
-	rlen = sprintf(buf, "version: " REL_VERSION " (api:%d/proto:%d)\n%s\n",
-		       API_VERSION,PRO_VERSION, drbd_buildtag());
+	seq_printf(seq, "version: " REL_VERSION " (api:%d/proto:%d)\n%s\n",
+		    API_VERSION,PRO_VERSION, drbd_buildtag());
 
 	/*
 	  cs .. connection state
@@ -216,10 +253,9 @@ int drbd_proc_get_info(char *buf, char **start, off_t offset,
 				sn = "ServerForDLess";
 		}
 		if ( drbd_conf[i].cstate == Unconfigured )
-			rlen += sprintf( buf + rlen,
-			   "%2d: cs:Unconfigured\n", i);
+			seq_printf( seq, "%2d: cs:Unconfigured\n", i);
 		else
-			rlen += sprintf( buf + rlen,
+			seq_printf( seq,
 			   "%2d: cs:%s st:%s/%s ld:%s\n"
 			   "    ns:%u nr:%u dw:%u dr:%u al:%u bm:%u "
 			   "lo:%d pe:%d ua:%d ap:%d\n",
@@ -244,12 +280,15 @@ int drbd_proc_get_info(char *buf, char **start, off_t offset,
 
 		if ( drbd_conf[i].cstate == SyncSource ||
 		     drbd_conf[i].cstate == SyncTarget )
-			rlen += drbd_syncer_progress(drbd_conf+i,buf+rlen);
+			drbd_syncer_progress(drbd_conf+i,seq);
 	}
 
-	/* DEBUG & profile stuff end */
+	return 0;
+}
 
-	return rlen;
+STATIC int drbd_proc_open(struct inode *inode, struct file *file)
+{
+	return seq_open(file, &drbd_proc_seq_ops);
 }
 
 /* PROC FS stuff end */
