@@ -19,6 +19,8 @@ enum {
 
   RETURN_PID           = 2,
   SLEEPS_FOREVER       = 4,
+
+  DONT_REPORT_FAILED   = 0x80,
 };
 
 /* for check_uniq(): Check for uniqueness of certain values...
@@ -28,9 +30,11 @@ enum {
 /* for verify_ips(): make not verifyable ips fatal */
 //#define INVALID_IP_IS_INVALID_CONF
 
+
 struct d_globals
 {
   int disable_io_hints;
+  int disable_ip_verification;
   int minor_count;
   int dialog_refresh;
 };
